#include "gl_camera.h"

#include <GL/gl.h>

void gl_camera::set_elevation(double deg)
{
    double new_deg = deg;

    // get the remainder
    new_deg = new_deg - (int)(new_deg / 360.0) * 360.0;

    if (new_deg > 0)
    {
        if (new_deg > 180.0)
            new_deg = new_deg - 360;
    }
    else
    {
        if (new_deg < -180.0)
            new_deg =  new_deg + 360;

    }
    
    this->elevation = new_deg;
}

double gl_camera::get_elevation() const
{
    return this->elevation;
}

void gl_camera::set_delta_elevation(double deg)
{
    this->set_elevation(this->elevation + deg);
}

// the number of degrees about the z-axis
void gl_camera::set_twist(double deg)
{
    double new_deg = deg;

    // get the remainder
    new_deg = new_deg - (int)(new_deg / 360.0) * 360.0;

    if (new_deg > 0)
    {
        if (new_deg > 180.0)
            new_deg = new_deg - 360;
    }
    else
    {
        if (new_deg < -180.0)
            new_deg =  new_deg + 360;

    }
    
    this->twist = new_deg;
}

void gl_camera::set_delta_twist(double delta_deg)
{
    this->set_twist(this->get_twist() + delta_deg);
}

double gl_camera::get_twist() const
{
    return this->twist;
}

void gl_camera::set_distance(double d)
{
    this->distance = d;
}

void gl_camera::set_delta_distance(double delta_d)
{
    this->set_distance(this->get_distance() + delta_d);
}

double gl_camera::get_distance()
{
    return this->distance;
}

void gl_camera::set_focal_point(const point3& focal_point)
{
    this->focal_point = focal_point;
}

const point3& gl_camera::get_focal_point()
{
    return this->focal_point;
}

void gl_camera::apply_gl_transform()
{

    glTranslated(0.0, 0.0, -this->distance);
    glRotated(-this->get_elevation(), -1.0, 0.0, 0.0);
    glRotated(-this->get_twist(), 0.0, 1.0, 0.0);
    glTranslated(
        -this->focal_point[0], -this->focal_point[1], -this->focal_point[2]
    );
}
